/*------------------------------------------------------------------------------
--  This file is a part of the SocExplorer Software
--  Copyright (C) 2012, Laboratory of Plasma Physics - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Paul LEROY
--                     Mail : paul.leroy@lpp.polytechnique.fr
----------------------------------------------------------------------------*/
#include <QHostAddress>
#include <QIODevice>
#include <QApplication>
#include <QTimer>
#include <socexplorerproxy.h>
#include <PythonQt.h>
#include "lfrcontrolplugin.h"

lfrcontrolplugin::lfrcontrolplugin(QWidget *parent)
  :socexplorerplugin(parent,false)
{
    //==========================================================================
    //PyWrapper CRAP
    timer = new QTimer;
    timer->setSingleShot(true);
    connect(timer, SIGNAL(timeout()), this, SLOT(sendTC()));
    connect(this, SIGNAL(activateTCLoopSig()), this, SLOT(sendTCLoop()));
    //==========================================================================

    this->UI = new lfrcontrolPluginUI();
    this->setWindowTitle(tr("RMAP and SPW Communication"));
    this->setWidget((QWidget*)this->UI);

    scanDone = false;
    timeCode = 0;
    time_COARSE = 0;
    time_FINE = 0;
    currentBridge = selectedBridgeIsUnknown;

    connectPythonWrapper();

    //**************
    // get a smart pointer to the __main__ module of the Python interpreter
    PythonQtObjectPtr context = PythonQt::self()->getMainModule();
    // add a QObject as variable of name "BUTTON_rmapOpenCommunication" to the namespace of the __main__ module
    context.addObject("BUTTON_rmapOpenCommunication", UI->rmapOpenCommunicationButton);
    context.addObject("BUTTON_rmapCloseCommunication", UI->rmapCloseCommunicationButton);
    context.addObject("BUTTON_selectStarDundee", UI->selectStarDundee_BUTTON);
    context.addObject("BUTTON_selectGRESB", UI->selectGRESB_BUTTON);
    //**************

    connect(this, SIGNAL(
                updateStatistics(unsigned char,unsigned char,unsigned char,unsigned char,uint,uint,uint,uint)
                ),
            this->UI->tmStatistics, SLOT(
                updateStatistics(unsigned char,unsigned char,unsigned char,unsigned char,uint,uint,uint,uint)
                ));

    connect( UI->rmapOpenCommunicationButton, SIGNAL(clicked()), this, SLOT(openBridge()));
    connect( UI->rmapCloseCommunicationButton, SIGNAL(clicked()),this, SLOT(closeBridge()));
    connect( this->UI, SIGNAL(sendMessage(QString)),         this, SLOT(displayOnConsole(QString)));
    connect( this->UI, SIGNAL(sendPacket(TMPacketToRead*)),  this, SLOT(receivePacketFromBridge(TMPacketToRead*)), Qt::DirectConnection);

    //***************
    // Python Wrapper
    connect(this, SIGNAL(TCPServerConnect()),this->UI,SLOT(connectServer()));
    // TODO rename setSpwServerIP to SetSpwServerIP
    connect( this, SIGNAL(SetSpwServerIP(unsigned char,unsigned char,unsigned char,unsigned char)),
             this->UI, SLOT(setSpwServerIP(unsigned char,unsigned char,unsigned char,unsigned char)) );
    connect(this, SIGNAL(TMEchoBridgeOpenPort()),this->UI->tmEchoBridge, SIGNAL(sig_listenOnTMEchoBridgePort()) );
    connect( this, SIGNAL(getNbTransmittedPackets()),this->UI, SLOT(getNbTransmittedPackets()) );
//    ((lfrcontrolpluginPythonWrapper*)this->pyObject)->ccsdsPacketStore = &(this->generalCCSDSPacketStore);

    //************
    // LFR Actions
    connect(this->UI->lfrActions, SIGNAL(WriteSPWSig(char*,uint,char,char)),
            this, SLOT(WriteSPW(char*,uint,char,char)));

}

lfrcontrolplugin::~lfrcontrolplugin()
{
    this->UI->tmStatistics->writeSettings();
}

unsigned int lfrcontrolplugin::Write(unsigned int *Value, unsigned int count, unsigned int address)
{
    unsigned int result = 1;

    return result;
}

unsigned int lfrcontrolplugin::Read(unsigned int *Value, unsigned int count, unsigned int address)
{
    unsigned int result = 1;

    return result;
}

////////
// SLOTS

unsigned int lfrcontrolplugin::WriteSPW( char *Value, unsigned int count, char targetLogicalAddress, char userApplication ) // SLOT
{
    unsigned int result = 1;
    char *data;
    unsigned int i;

    data = (char*) malloc(count + 4);
    data[0] = 0x00;   // Protocole = 0 => Host to SpaceWire packet transmission
    data[1] = (unsigned char) (count >> 16);
    data[2] = (unsigned char) (count >> 8 );
    data[3] = (unsigned char) (count      );
    for (i=0; i<count; i++)
    {
        data[i+4] = Value[i];
    }

    if (this->UI->socketSpwServer->isOpen())
    {
        this->UI->socketSpwServer->write( data, count + 4);
        this->UI->onePacketTransmitted( count + 4 );
    }

    free(data);

    return result;
}

void lfrcontrolplugin::openBridge()
{
    emit activateSig( this->Connected );
}

void lfrcontrolplugin::closeBridge()
{
    this->Connected = false;
    emit activateSig( this->Connected );
}

void lfrcontrolplugin::RMAP_write_reply_setText(QString text)
{
    this->UI->RMAP_write_reply->setText(text);
}

void lfrcontrolplugin::appendToLog(QString text)
{
    APPENDTOLOG(text);
}

void lfrcontrolplugin::setValueTargetAddress(unsigned char newAddress)
{
    this->UI->rmapTargetLogicalAddressSpinBox->setValue(newAddress);
}

void lfrcontrolplugin::setValueSourceAddress(unsigned char newAddress)
{
    this->UI->rmapSourceLogicalAddressSpinBox->setValue(newAddress);
}

void lfrcontrolplugin::setValueSpaceWireLinkNumber(unsigned char linkNumber)
{
//    this->UI->spwLinkSpinBox->setValue( linkNumber );
    emit this->spaceWireLinkNumberHasChanged( linkNumber );
}

void lfrcontrolplugin::receivePacketFromBridge(TMPacketToRead *packet)
{
    preProcessPacket(packet);

    // Send the packet to the TM echo bridge for processing
    this->UI->tmEchoBridge->sendTMPacketLESIA(packet);

    this->generalCCSDSPacketStore.append(packet);
    if (this->generalCCSDSPacketStore.size() > 200)
    {
        this->generalCCSDSPacketStore.erase(generalCCSDSPacketStore.begin(),
                                            generalCCSDSPacketStore.begin() + 100);
    }
    this->UI->nbPacketInStore->setText("nb packets in store: " + QString::number(generalCCSDSPacketStore.size()));
    processPacketStore();
}

void lfrcontrolplugin::preProcessPacket(TMPacketToRead *packet)
{
    unsigned char pid = 0;
    unsigned char cat = 0;
    unsigned char typ = 0;
    unsigned char sub = 0;
    unsigned int sid = 0;
    unsigned int length = 0;
    unsigned int coarse_t = 0;
    unsigned int fine_t = 0;

    //*********************************
    // get the parameters of the packet
    pid         = this->UI->tmStatistics->getPID( packet );
    cat         = this->UI->tmStatistics->getCAT( packet );
    typ         = this->UI->tmStatistics->getTYPE( packet );
    sub         = this->UI->tmStatistics->getSUBTYPE( packet );
    length      = this->UI->tmStatistics->getLENGTH( packet );
    coarse_t    = this->UI->tmStatistics->getCoarseTime( packet );
    fine_t      = this->UI->tmStatistics->getFineTime( packet );
    sid         = this->UI->tmStatistics->getSID( packet, pid, cat, typ, sub );

    emit updateStatistics(pid, cat, typ, sub, sid, length, coarse_t, fine_t);

    //**********************************************
    // if the packet is a parameter dump, display it
    if ( (typ == TM_TYPE_PARAMETER_DUMP) & (sub == TM_SUBTYPE_PARAMETER_DUMP) & (sid == SID_PARAMETER_DUMP) )
        this->UI->lfrActions->parameterDump->updateParameterDump(packet);
    //**********************************************
    // if the packet is a kcoefficients dump, display it
    if ( (typ == TM_TYPE_PARAMETER_DUMP) & (sub == TM_SUBTYPE_K_DUMP) & (sid == SID_K_DUMP) )
        this->UI->lfrActions->parameterDump->processKCoefficientsDump(packet);
    //********************************************************
    // if the packet is an HK packet, get the software version
    if ( (typ == TM_TYPE_HK) & (sub == TM_SUBTYPE_HK) & (sid == SID_HK) )
        this->UI->tmStatistics->updateSWVersion( packet );
}

void lfrcontrolplugin::nbPacketHasChanged(int nb)
{
    this->UI->nbPacketInStore->setText("nb packets in store: " + QString::number(nb));
}

/////////////////////
// INTERNAL FUNCTIONS

void lfrcontrolplugin::processCCSDSPacket(unsigned char *ccsdsPacket, unsigned int size) // SLOT
{
    QString message;
    unsigned int fine_time_value = 0;
    fine_time_value = ((unsigned int) ccsdsPacket[7]<<24)
            + ((unsigned int) ccsdsPacket[6]<<16)
            + ((unsigned int) ccsdsPacket[5]<<8)
            + ((unsigned int) ccsdsPacket[4]);
    message.append(QTime::currentTime().toString() +":" + QString::number(QTime::currentTime().msec()) + ": ");
    message.append("size "
                   + QString::number(size)
                   +" *** header "
                   + QString::number(ccsdsPacket[0], 16)
                   + " "
                   + QString::number(ccsdsPacket[1], 16)
                   + " "
                   + QString::number(ccsdsPacket[2], 16)
                   + " "
                   + QString::number(ccsdsPacket[3], 16)
                   + " *** coarse time "
                   + QString::number(fine_time_value));
                   //+ QString::number(ccsdsPacket[4], 16)
                   //+" "
                   //+ QString::number(ccsdsPacket[5], 16)
                   //+" "
                   //+ QString::number(ccsdsPacket[6], 16)
                   //+" "
                   //+ QString::number(ccsdsPacket[7], 16));
    displayOnConsole(message);
    //((lfrcontrolpluginPythonWrapper*)this->pyObject)->storeCCSDSPacket(ccsdsPacket, size);
    emit ccsdsPacketIsProcessed();
}

int lfrcontrolplugin::fetchPacket()
{
    int ret = 0;

    return ret;
}

void lfrcontrolplugin::connectPythonWrapper()
{
//    this->pyObject = new lfrcontrolpluginPythonWrapper(this);
    //**
//    connect( this, SIGNAL( WriteSPWSig(char*,uint,char,char) ),
//            this, SLOT( WriteSPW(char*,uint,char,char) ), Qt::DirectConnection );
    //**
    connect( this, SIGNAL( updateSourceLogicalAddress(unsigned char) ),
            this, SLOT( setValueSourceAddress(unsigned char) ) );
    //**
    connect( this, SIGNAL( updateSpaceWireLinkNumber(unsigned char) ),
            this, SLOT( setValueSpaceWireLinkNumber(unsigned char) ) );
    //**
    connect( this, SIGNAL(sendMessage(QString)),
             this, SLOT(displayOnConsole(QString)) );
    //**
    connect( this, SIGNAL(fetchPacketSig()),
             this, SLOT(fetchPacket()), Qt::DirectConnection );
    //***
//    connect( this, SIGNAL(nbPacketHasChanged(int)),
//             this, SLOT(nbPacketHasChanged(int)));
    //******
    //RECORD
    connect( this, SIGNAL( set_logPackets(bool) ),
             this, SLOT( logPackets(bool) ) );
    connect( this, SIGNAL( set_storePacketsRAW(bool) ),
             this, SLOT( storePacketsRAW(bool) ) );
    connect( this, SIGNAL( set_storePacketsCSV(bool) ),
             this, SLOT( storePacketsCSV(bool) ) );
}

void lfrcontrolplugin::logPackets( bool state )
{
    this->UI->tmStatistics->checkbox_packetLog->setChecked( state );
}

void lfrcontrolplugin::storePacketsRAW(bool state , QString fileName)
{
    this->UI->tmStatistics->storePackets(state, fileName);
}

void lfrcontrolplugin::storePacketsCSV( bool state )
{
    this->UI->tmStatistics->checkbox_CSVPacketRecording->setChecked( state );
}

QString lfrcontrolplugin::getSoftwareVersion()
{
    QString version;

    version = QString::number(SW_VERSION_N1) + "."
            + QString::number(SW_VERSION_N2) + "."
            + QString::number(SW_VERSION_N3) + "."
            + QString::number(SW_VERSION_N4);

    return version;
}

QList<QVariant> lfrcontrolplugin::getFlightSoftwareVersion()
{
    return this->UI->getSWVersion();
}

void lfrcontrolplugin::generatePDFFromQTextDocument(QTextDocument *document, QString filename)
{
    QPrinter *printer;
    printer = new QPrinter( QPrinter::HighResolution);
    printer->setOutputFormat( QPrinter::PdfFormat);
    printer->setOutputFileName( filename );

    document->print( printer );
}

void lfrcontrolplugin::ProcessPendingEvents()
{
    QCoreApplication::processEvents();
}

void lfrcontrolplugin::setValueTargetLogicalAddress(unsigned int address)
{
    unsigned char newAddress;
    newAddress = (unsigned char) address;
    // TODO Not used?
    emit updateTargetLogicalAddress(newAddress);
}

void lfrcontrolplugin::setValueSourceLogicalAddress(unsigned int address)
{
    unsigned char newAddress;
    newAddress = (unsigned char) address;
    this->setValueSourceAddress(newAddress);
//    emit updateSourceLogicalAddress(newAddress);
}

void lfrcontrolplugin::setValueSpaceWireLinkNumber(unsigned int linkNumber)
{
    unsigned char newLinkNumber;
    newLinkNumber = (unsigned char) linkNumber;
    this->setValueSpaceWireLinkNumber(linkNumber);
//    emit updateSpaceWireLinkNumber(newLinkNumber);
}

void lfrcontrolplugin::sendTCLoop()
{
    unsigned int delay;

    if (!timer->isActive())
    {
        if (!TCPacketStore.isEmpty())
        {
            delay = TCPacketStore.at(0)->delay;
            timer->setInterval(delay);
            timer->start();
        }
    }
}

void lfrcontrolplugin::sendTC()
{
    TCPacketToSend *packet;
    unsigned int delay;

    packet = TCPacketStore.takeFirst();
    // the signal is connected with the option Qt::DirectConnection, thus it is processed immediately
//    emit(WriteSPWSig(packet->Value, packet->count,packet->targetLogicalAddress, packet->userApplication));
    WriteSPW(packet->Value, packet->count,packet->targetLogicalAddress, packet->userApplication);
    delete(packet);
    if (!TCPacketStore.isEmpty())
    {
        delay = TCPacketStore.at(0)->delay;
        timer->setInterval(delay);
        timer->start();
    }
}


void lfrcontrolplugin::processPacketStoreLater(unsigned int delay)
{
    displayOnConsole(
                "*** process packet store in " + QString::number(delay) + " ms"
                );
    QTimer::singleShot(delay, this, SLOT(sendProcessPacketStoreNowSig()));
}



void lfrcontrolplugin::processPacketStore()
{
    emit ccsdsPacketIsAvailable(generalCCSDSPacketStore.at(0)->size);
}


void lfrcontrolplugin::WriteSPW(QList<QVariant> dataList)
{
    char targetLogicalAddress;
    char userApplication;
    unsigned int count;
    char *Value;

    unsigned char data[dataList.count()];

    for(int i = 0; i<dataList.count(); i++) // get the data as unsigned char
    {
        data[i] = (unsigned char) dataList.at(i).toUInt();
    }
    // read the first bytes of the data to get the targetLogicalAddress and the userApplication parameters
    targetLogicalAddress = data[0];
    userApplication = data[3];

    count = dataList.count() - 4; // the CCSDS header (4 bytes) is added by LPPMON
    Value = (char*) &data[4];

//    emit WriteSPWSig(Value, count, targetLogicalAddress, userApplication);
    WriteSPW(Value, count, targetLogicalAddress, userApplication);
}

void lfrcontrolplugin::WriteSPWDelay(QList<int> dataList, unsigned int delay)
{
    char targetLogicalAddress;
    char userApplication;
    unsigned int count;
    char *Value;
    TCPacketToSend *packet;

    unsigned char data[dataList.count()];

    for(int i = 0; i<dataList.count(); i++) // get the data as unsigned char
    {
        data[i] = (unsigned char)dataList.at(i);
    }
    // read the first bytes of the data to get the targetLogicalAddress and the userApplication parameters
    targetLogicalAddress = data[0];
    userApplication = data[3];

    count = dataList.count() - 4; // the CCSDS header (4 bytes) is added by LPPMON
    Value = (char*) &data[4];

    packet = new TCPacketToSend(Value, count, targetLogicalAddress, userApplication, delay);
    TCPacketStore.append(packet);

    emit(activateTCLoopSig());

}


QList<QVariant> lfrcontrolplugin::ReadSPW(unsigned int size) // size is not used
{
    Q_UNUSED(size)
    QList<QVariant> result;
    TMPacketToRead *ccsdsPacket;

    if (!generalCCSDSPacketStore.isEmpty())
    {
        ccsdsPacket = generalCCSDSPacketStore.takeFirst();
        for(unsigned int i = 0;i<ccsdsPacket->size;i++)
        {
            result.append((QVariant)ccsdsPacket->Value[i]);
        }
        delete(ccsdsPacket);
    }
    nbPacketHasChanged(generalCCSDSPacketStore.size());
    return result;
}


