/*------------------------------------------------------------------------------
--  This file is a part of the LPPMON Software
--  Copyright (C) 2011, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@lpp.polytechnique.fr
----------------------------------------------------------------------------*/
#ifndef DRIVER1_H
#define DRIVER1_H

#include <QMenuBar>
#include <QMenu>
#include <QAction>
#include <QMainWindow>
#include <QTcpSocket>
#include <QTime>
#include <ccsds.h>
#include <QSemaphore>
#include <QPrinter>

#include <socexplorerplugin.h>

#include "rmapoperations.h"
#include "ccsds.h"
#include "tcpackettosend.h"
#include "tmpackettoread.h"

#include "lfrcontrolpluginui.h"
#include "rmapoperations.h"

#define APPENDTOLOG(message) this->UI->appendToLogFile(QTime::currentTime().toString() +":" + QString::number(QTime::currentTime().msec()) + ": " + message)

#define READ_WRITE_MAX_COUNTS 4096                      // in words
#define RMAP_MAX_DATA_LENGTH 4*READ_WRITE_MAX_COUNTS    // in bytes, shall be lower than the limit size of SPW packet of the GRESB bridge
#define RMAP_READ_REPLY_HEADER_LENGTH 12                // in bytes => ECSS
#define RMAP_READ_COMMAND_HEADER_LENGTH 16              // in bytes => ECSS
#define RMAP_DATA_CRC_LENGTH 1                          // in bytes => ECSS
#define CCSDS_MAX_PACKET_LENGTH 4412                    // in bytes, specified in the LFR Software Requirements Specification

class lfrcontrolplugin : public socexplorerplugin
{
    Q_OBJECT
public:
    explicit lfrcontrolplugin(QWidget *parent = 0);
    ~lfrcontrolplugin();

    void preProcessPacket(TMPacketToRead *packet);
    void connectPythonWrapper();

    QList<TMPacketToRead*> generalCCSDSPacketStore;


    //From PyWrapper
    unsigned int storeCCSDSPacket(unsigned char *ccsdsPacket, unsigned int size);

//    QList<TMPacketToRead*> *ccsdsPacketStore;
    QList<TCPacketToSend*> TCPacketStore;
    QTimer *timer;

public slots:
    unsigned int Write(unsigned int *Value = NULL,unsigned int count=0,unsigned int address=0);
    unsigned int Read(unsigned int *Value = NULL,unsigned int count=0,unsigned int address=0);
    unsigned int WriteSPW(char *Value = NULL, unsigned int count=0, char targetLogicalAddress=0, char userApplication=0);
    //
    void setValueTargetAddress(unsigned char newAddress);
    void setValueSourceAddress(unsigned char newAddress);
    void setValueSpaceWireLinkNumber(unsigned char linkNumber);
    //
    void openBridge();
    void closeBridge();
    void processCCSDSPacket(unsigned char *ccsdsPacket, unsigned int size);
    void processPacketStore();
    void nbPacketHasChanged(int nb);
    int fetchPacket();
    void receivePacketFromBridge(TMPacketToRead* packet);
    //
    void displayOnConsole(QString message) {this->UI->console->append(message);}
    //
    void RMAP_write_reply_setText(QString text);
    void appendToLog(QString text);
    //
    void activatePlugin(bool flag) {emit activateSig(flag);}
    void bridgeHasChanged(selectedBridge bridge) {currentBridge = bridge;}
    // RECORD
    void logPackets( bool state );
    void storePacketsRAW( bool state ,QString fileName="");
    void storePacketsCSV( bool state );

    //moved slots from pywrapper
    QList<QVariant> ReadSPW(unsigned int size=0);
//    int fetchPacket() {return emit fetchPacketSig();}
    void WriteSPW(QList<QVariant> dataList);
    void WriteSPWDelay(QList<int> dataList, unsigned int delay=0);
    void sendTCLoop();
    void sendTC();
    void reEmitWriteSPWSig(char *Value, unsigned int count, char targetLogicalAddress, char userApplication)
        { WriteSPW(Value, count, targetLogicalAddress, userApplication);}
    void setValueTargetLogicalAddress(unsigned int address);
    void setValueSourceLogicalAddress(unsigned int address);
    void setValueSpaceWireLinkNumber(unsigned int linkNumber);
    void ProcessPendingEvents();
    void generatePDFFromQTextDocument(QTextDocument *document , QString filename);
    QString getSoftwareVersion();
    QList<QVariant> getFlightSoftwareVersion();
    void processPacketStoreLater(unsigned int delay);
    void sendProcessPacketStoreNowSig(){emit processPacketStoreNowSig();}
signals:
    void ccsdsPacketIsProcessed();
    void updateStatistics(unsigned char pid, unsigned char cat,
                          unsigned char typ, unsigned char sub,
                          unsigned int sid, unsigned int length,
                          unsigned int coarse_t, unsigned int fine_t);
    void spaceWireLinkNumberHasChanged( unsigned char );

    // Signals from py wrapper
    void SetSpwServerIP( unsigned char address1, unsigned char address2, unsigned char address3, unsigned char address4 );
    void TCPServerConnect();
    void ccsdsPacketIsAvailable(unsigned int size);
    void processPacketStoreNowSig();
    void TMEchoBridgeOpenPort( void );
    void activateTCLoopSig();
    unsigned int getNbTransmittedPackets();
    void sendMessage(QString messageToDisplay);
    int fetchPacketSig();
    void set_logPackets(bool);
    void set_storePacketsRAW(bool);
    void set_storePacketsCSV(bool);
    // TODO used?
    void updateTargetLogicalAddress(unsigned char newAddress);
    void updateSourceLogicalAddress(unsigned char newAddress);
    void updateSpaceWireLinkNumber(unsigned char newAddress);
private:
    selectedBridge currentBridge;
    lfrcontrolPluginUI* UI;
    ccsds* ccsds_command;
    unsigned int rmapPacketSize;
    unsigned int time_COARSE;
    unsigned int time_FINE;
    char timeCode;
    bool scanDone;
};

#endif // DRIVER1_H
