/*------------------------------------------------------------------------------
--  This file is a part of the LPPMON Software
--  Copyright (C) 2011, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@lpp.polytechnique.fr
----------------------------------------------------------------------------*/
#include "lfrcontrolplugin.h"
#include <socexplorerengine.h>

lfrcontrolPluginUI::lfrcontrolPluginUI(QWidget *parent) :
    QWidget(parent)
{
    incomingTMParser = new IncomingTMParser();
    spwTabWidget = new QTabWidget;
    //
    QWidget* spwTabWidgetPage0 = new QWidget;
    QWidget* spwTabWidgetPage3 = new QWidget;
    QWidget* spwTabWidgetPage4 = new QWidget;
    QWidget* spwTabWidgetPage6 = new QWidget;
    //
    bridgeWidget = new QWidget;
    mainLayout = new QVBoxLayout;
    ccsdsLayout = new QVBoxLayout;
    consoleLayout = new QVBoxLayout;
    generalParameters_LAYOUT = new QGridLayout;

    buildSelectionLayout();

    //*** QLABEL ***//
    gresbBridgeIPLabel = new QLabel(tr("Bridge IP: "));
    gresbVirtualLinkLabel = new QLabel(tr("Virtual Link: "));
    spwLinkLabel = new QLabel(tr("SPW Link: "));
    rmapSourceLogicalAddressLabel = new QLabel(tr("RMAP Source Logical Address: "));
    rmapTargetLogicalAddressLabel = new QLabel(tr("RMAP Target Logical Address: "));
    logFileName = new QLabel;
    gresbStatusQueryLabel = new QLabel(tr("Status query socket (port 3010): waiting for connection"));
    gresbStatusQueryDialogLabel = new QLabel(tr("sockets opened but SpaceWire link not running"));
    nbPacketInStore = new QLabel(tr("nb packets in store: -"));

    //*** QPUSHBUTTON ***//
    rmapOpenCommunicationButton = new QPushButton(tr("Open selected bridge"));
    rmapCloseCommunicationButton = new QPushButton(tr("Close selected bridge"));
    rmapOpenCommunicationButton->setEnabled(false);
    rmapCloseCommunicationButton->setEnabled(false);
    logFileChooseButton = new QPushButton(tr("Choose file"));
    gresbStatusQueryRetryButton = new QPushButton(tr("Retry"));
    gresbStatusQueryAbortButton = new QPushButton(tr("Abort"));
    clearConsoleButton = new QPushButton(tr("Clear"));

    selectGRESB_BUTTON = new QRadioButton(tr("GRESB"));
    selectStarDundee_BUTTON = new QRadioButton(tr("Star Dundee"));

    //*** SPINBOX ***//
    gresbVirtualLinkSpinBox = new QSpinBox;
    rmapSourceLogicalAddressSpinBox = new QSpinBox;
    rmapTargetLogicalAddressSpinBox = new QSpinBox;
    CCSDSTargetLogicalAddressSpinBox = new QSpinBox;
    gresbVirtualLinkSpinBox->setRange(0, 4);
    gresbVirtualLinkSpinBox->setValue(1);
    rmapSourceLogicalAddressSpinBox->setRange(0, 255);
    rmapTargetLogicalAddressSpinBox->setRange(0, 255);

    //*** MISC **//
    tmEchoBridge = new TMEchoBridge();
    console = new QTextEdit;
    gresbStatusQueryDialog = new QDialog;
    logEnableCheckBox = new QCheckBox(tr("Enable Logs"));
    RMAP_write_verify = new QCheckBox(tr("data checked before write\nlimited to 4 bytes\nNOT IMPLEMENTED"));
    RMAP_write_reply = new QCheckBox(tr("reply to the write command required\nlast reply status: unavailable"));
    logFile = new QFile();
    tmStatistics = new TMStatistics();
    lfrActions = new LFRActions();

    logFileEn = false;

    //********
    // CONSOLE
    consoleLayout->addWidget(console);
    consoleLayout->addWidget(clearConsoleButton);
    consoleLayout->addWidget(nbPacketInStore);

    connect(this->logFileChooseButton, SIGNAL(clicked()), this, SLOT(chooseLogFile()));
    connect(this, SIGNAL(setLogFileName(QString)), this->logFileName, SLOT(setText(QString)));
    connect(this->logEnableCheckBox, SIGNAL(stateChanged(int)), this, SLOT(logFileEnDisable(int)));

    spwTabWidget->addTab(spwTabWidgetPage0, tr("connection"));
    spwTabWidget->addTab(spwTabWidgetPage3, tr("console"));
    spwTabWidget->addTab(spwTabWidgetPage4, tr("TM statistics"));
    spwTabWidget->addTab(lfrActions, tr("dashboard"));
    spwTabWidget->addTab(spwTabWidgetPage6, tr("TM Echo Bridge"));

    spwTabWidgetPage0->setLayout(selectionLayout);
    spwTabWidgetPage3->setLayout(consoleLayout);
    spwTabWidgetPage4->setLayout(tmStatistics->layout());
    spwTabWidgetPage6->setLayout(tmEchoBridge->layout());

    mainLayout->addWidget(spwTabWidget);
    setLayout(mainLayout);

    connect(this->clearConsoleButton, SIGNAL(clicked()), this->console, SLOT(clear()));

    // command code
    connect(this->RMAP_write_reply, SIGNAL(clicked()), this, SLOT(getCommandCode()));
    connect(this->RMAP_write_verify, SIGNAL(clicked()), this, SLOT(getCommandCode()));

    connect(this->tmStatistics, SIGNAL(lastTMTimeHasChanged(uint)),
            this->lfrActions->enterMode, SLOT(lastTMTimeHasChanged(uint)));

    getCommandCode(); // init the command code value
    rmapSourceLogicalAddressSpinBox->setValue(DEFAULT_SOURCE);
    rmapTargetLogicalAddressSpinBox->setValue(DEFAULT_TARGET);

    readSettings();
    this->spwServerIPDialogBox->setIP(echoBridgeIP0, echoBridgeIP1, echoBridgeIP2, echoBridgeIP3);

    connect(this->incomingTMParser, SIGNAL(sendPacket(TMPacketToRead*)),this, SIGNAL(sendPacket(TMPacketToRead*)));
    connect(this->incomingTMParser, SIGNAL(sendMessage(QString)),       this, SIGNAL(sendMessage(QString)));

}

lfrcontrolPluginUI::~lfrcontrolPluginUI()
{
    writeSettings();
    //delete ui;
}

//*************************
// <SpaceWire TC/TM server>
void lfrcontrolPluginUI::buildStatisticsLayout( void )
{
    statisticsLayout        = new QGridLayout;

    label_received          = new QLabel(tr("Received"));
    label_transmitted       = new QLabel(tr("Transmitted"));
    label_bytes             = new QLabel(tr("Bytes"));
    label_packets           = new QLabel(tr("Packets"));
    label_receivedBytes     = new QLabel(tr("-"));
    label_receivedPackets   = new QLabel(tr("-"));
    label_transmittedBytes  = new QLabel(tr("-"));
    label_transmittedPackets= new QLabel(tr("-"));

    button_resetStatistics  = new QPushButton(tr("Reset statistics"));

    statisticsLayout->addWidget(button_resetStatistics, 0, 0, 1, 1);
    statisticsLayout->addWidget(label_received,         0, 1, 1, 1);
    statisticsLayout->addWidget(label_transmitted,      0, 2, 1, 1);

    statisticsLayout->addWidget(label_packets,            1, 0, 1, 1);
    statisticsLayout->addWidget(label_receivedPackets,    1, 1, 1, 1);
    statisticsLayout->addWidget(label_transmittedPackets, 1, 2, 1, 1);

    statisticsLayout->addWidget(label_bytes,            2, 0, 1, 1);
    statisticsLayout->addWidget(label_receivedBytes,    2, 1, 1, 1);
    statisticsLayout->addWidget(label_transmittedBytes, 2, 2, 1, 1);

    statisticsLayout->setColumnStretch(3, 1);
    statisticsLayout->setRowStretch(3, 1);
}

void lfrcontrolPluginUI::buildSelectionLayout( void )
{
    consoleSpaceWireServer = new QTextEdit;
    socketSpwServer = new QTcpSocket();

    selection_GROUPBOX = new QGroupBox(tr("SpaceWire TC/TM server"));
    selectionLayout = new QVBoxLayout;
    layout_TMEcho = new QVBoxLayout;


    label_serverTMEcho          = new QLabel(tr("Server IP"));
    spwServerIPDialogBox        = new QIPDialogBox();
    label_serverTMEchoPort      = new QLabel(tr("Server Port"));
    spinbox_serverTMEchoPort    = new QSpinBox();
    button_openSocketEchoServer = new QPushButton(tr("Open"));

    initSocketStatesList();

    buildStatisticsLayout();

    spinbox_serverTMEchoPort->setRange(2000, 65535);
    spinbox_serverTMEchoPort->setValue(2200);

    layout_TMEcho->addWidget(label_serverTMEcho);
    layout_TMEcho->addWidget(spwServerIPDialogBox);
    layout_TMEcho->addWidget(label_serverTMEchoPort);
    layout_TMEcho->addWidget(spinbox_serverTMEchoPort);
    layout_TMEcho->addWidget(button_openSocketEchoServer);
    layout_TMEcho->addLayout(statisticsLayout);
    selection_GROUPBOX->setLayout(layout_TMEcho);

    selectionLayout->addWidget(selection_GROUPBOX);
    selectionLayout->addWidget(consoleSpaceWireServer);

    selectionLayout->addStretch(1);

    connect(this->button_openSocketEchoServer, SIGNAL(clicked()),
            this, SLOT(toggleServer()));
    connect(this->socketSpwServer, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
            this, SLOT(socket_TMEcho_ServerHasChanged()));
    connect(this->socketSpwServer, SIGNAL(readyRead()),
            this, SLOT(incomingDataFromSpaceWire_alt()));
    connect(this->button_resetStatistics, SIGNAL(clicked()),
            this, SLOT(resetstatistics()));

    resetstatistics();
}

void lfrcontrolPluginUI::toggleServer()
{
    if(this->socketSpwServer->isOpen())
    {
       disconnectServer();
    }
    else
    {
        connectServer();
    }
}

void lfrcontrolPluginUI::connectServer( void )
{
    socketSpwServer->connectToHost(this->spwServerIPDialogBox->getIP(),
                                    this->spinbox_serverTMEchoPort->value());
    this->button_openSocketEchoServer->setText("Close");
}

void lfrcontrolPluginUI::setSpwServerIP( unsigned char address1, unsigned char address2, unsigned char address3, unsigned char address4 )
{
    this->spwServerIPDialogBox->setIP( address1, address2, address3, address4 );
}

void lfrcontrolPluginUI::disconnectServer( void )
{
    this->button_openSocketEchoServer->setText("Open");
    this->socketSpwServer->disconnectFromHost();
}

void lfrcontrolPluginUI::socket_TMEcho_ServerHasChanged()
{
    displayOnConsole( "TM Echo Socket socket *** " + socketStates.at(socketSpwServer->state()) );
}

void lfrcontrolPluginUI::initSocketStatesList()
{
    socketStates.append("The socket is not connected");
    socketStates.append("The socket is performing a host name lookup");
    socketStates.append("The socket has started establishing a connection");
    socketStates.append("A connection is established");
    socketStates.append("The socket is bound to an address and port (for servers)");
    socketStates.append("The socket is about to close (data may still be waiting to be written)");
    socketStates.append("For internal use only");
}

void lfrcontrolPluginUI::incomingDataFromSpaceWire()
{
    TMPacketToRead *generalPacket;
    int nbBytesAvailable = 0;
    char *buffer;

    nbBytesAvailable = socketSpwServer->bytesAvailable();
    buffer = (char *) malloc(nbBytesAvailable);
    socketSpwServer->read(buffer, nbBytesAvailable);

    onePacketReceived( nbBytesAvailable );

    generalPacket = new TMPacketToRead( (unsigned char *) &buffer[4], nbBytesAvailable-4);

    emit sendPacket(generalPacket);

    free(buffer);
}

void lfrcontrolPluginUI::incomingDataFromSpaceWire_alt()
{
    int nbBytesAvailable = 0;
    QByteArray dataArray;

    nbBytesAvailable = socketSpwServer->bytesAvailable();
    dataArray.append( socketSpwServer->readAll() );

    onePacketReceived( nbBytesAvailable );

    incomingTMParser->processIncomingQByteArray( dataArray );

    dataArray.clear();
}

void lfrcontrolPluginUI::displayOnConsole(QString message)
{
    consoleSpaceWireServer->append(message);
}

void lfrcontrolPluginUI::resetstatistics()
{
    receivedBytes = 0;
    receivedPackets = 0;
    transmittedBytes = 0;
    transmittedPackets = 0;

    label_receivedBytes->setText(      QString::number(receivedBytes) );
    label_receivedPackets->setText(    QString::number(receivedPackets) );
    label_transmittedBytes->setText(   QString::number(transmittedBytes) );
    label_transmittedPackets->setText( QString::number(transmittedPackets) );
}

void lfrcontrolPluginUI::onePacketTransmitted( unsigned int nbBytes )
{
    transmittedBytes = transmittedBytes + nbBytes;
    transmittedPackets = transmittedPackets + 1;

    label_transmittedBytes->setText(   QString::number(transmittedBytes) );
    label_transmittedPackets->setText( QString::number(transmittedPackets) );
}

unsigned int lfrcontrolPluginUI::getNbTransmittedPackets()
{
    return this->transmittedPackets;
}

QList<QVariant> lfrcontrolPluginUI::getSWVersion()
{
    QList<unsigned char> versionAsUnsignedChar;
    QList<QVariant> versionAsQVariant;

    versionAsUnsignedChar = this->tmStatistics->getSWVersion();

    if (versionAsUnsignedChar.size()!=4)
    {
        versionAsQVariant.append( (QVariant) ( (unsigned int) 0 ) );
        versionAsQVariant.append( (QVariant) ( (unsigned int) 0 ) );
        versionAsQVariant.append( (QVariant) ( (unsigned int) 0 ) );
        versionAsQVariant.append( (QVariant) ( (unsigned int) 0 ) );
    }
    else
    {
        versionAsQVariant.append( (QVariant) ( (unsigned int) versionAsUnsignedChar.at(0) ) );
        versionAsQVariant.append( (QVariant) ( (unsigned int) versionAsUnsignedChar.at(1) ) );
        versionAsQVariant.append( (QVariant) ( (unsigned int) versionAsUnsignedChar.at(2) ) );
        versionAsQVariant.append( (QVariant) ( (unsigned int) versionAsUnsignedChar.at(3) ) );
    }

    return versionAsQVariant;
}

void lfrcontrolPluginUI::onePacketReceived( unsigned int nbBytes )
{
    receivedBytes = receivedBytes + nbBytes;
    receivedPackets = receivedPackets + 1;

    label_receivedBytes->setText(   QString::number(receivedBytes) );
    label_receivedPackets->setText( QString::number(receivedPackets) );

    label_receivedBytes->update();
    label_receivedPackets->update();
}

// </ SpaceWire TC/TM server>
//***************************

void lfrcontrolPluginUI::chooseLogFile()
{
    if(this->logFile->isOpen())
        this->logFile->close();
    this->logFile->setFileName(QFileDialog::getSaveFileName(this,tr("Open Log file"),
                                                            QDir::homePath()
                                                            + "/"
                                                            + QDate::currentDate().toString()
                                                            + "_"
                                                            + QTime::currentTime().toString()
                                                            + "_rmapPluginUI.log",
                                                            tr("Log Files (*.txt *.log)")));
    if(this->logFile->open(QIODevice::WriteOnly))
    {
        this->logFileStrm = new QTextStream(this->logFile);
        emit this->setLogFileName(this->logFile->fileName());
    }
}

void lfrcontrolPluginUI::logFileEnDisable(int state)
{
    if(state==Qt::Checked)
    {
        this->logFileEn = true;
    }
    else if(state==Qt::Unchecked)
    {
        this->logFileEn = false;
    }
}

bool lfrcontrolPluginUI::islogfileenable()
{
    return this->logFileEn;
}

void lfrcontrolPluginUI::appendToLogFile(const QString & text)
{
    if(this->logFileEn && this->logFile->isOpen())
    {
        *(this->logFileStrm) << text << endl;
    }
}

void lfrcontrolPluginUI::closeEvent(QCloseEvent *event)
{
    if(this->logFile->isOpen())
    {
        this->logFileStrm->flush();
        this->logFile->waitForBytesWritten(3000);
        this->logFile->close();
    }
    event->accept();
}

RMAP_command_codes lfrcontrolPluginUI::getCommandCode()
{
    RMAP_command_codes commandCode = invalid0;
    if (RMAP_write_verify->isChecked() and RMAP_write_reply->isChecked()) commandCode = writeSingle_ver_rep;
    if (RMAP_write_verify->isChecked() and !RMAP_write_reply->isChecked()) commandCode = writeSingle_ver_noRep;
    if (!RMAP_write_verify->isChecked() and RMAP_write_reply->isChecked()) commandCode = writeSingle_noVer_Rep;
    if (!RMAP_write_verify->isChecked() and !RMAP_write_reply->isChecked()) commandCode = writeSingle_noVer_noRep;
    emit commandCodeHasChanged(commandCode);
    return commandCode;
}

// SLOT

void lfrcontrolPluginUI::isOpen(bool flag)
{
    if (flag == true)
    {
        selection_GROUPBOX->setEnabled(false);
        this->rmapOpenCommunicationButton->setEnabled(false);
        this->rmapCloseCommunicationButton->setEnabled(true);
    }
    if (flag == false)
    {
        selection_GROUPBOX->setEnabled(true);
        this->rmapOpenCommunicationButton->setEnabled(true);
        this->rmapCloseCommunicationButton->setEnabled(false);
    }
}

void lfrcontrolPluginUI::readSettings()
{
    QSettings settings("socexplorer", "lfrcontrolpluginui");
    echoBridgeIP0 = (unsigned char) settings.value("echoBridgeIP0", 127).toInt();
    echoBridgeIP1 = (unsigned char) settings.value("echoBridgeIP1", 0).toInt();
    echoBridgeIP2 = (unsigned char) settings.value("echoBridgeIP2", 0).toInt();
    echoBridgeIP3 = (unsigned char) settings.value("echoBridgeIP3", 1).toInt();
}

void lfrcontrolPluginUI::writeSettings()
{
    QSettings settings("socexplorer", "lfrcontrolpluginui");
    settings.setValue("echoBridgeIP0", this->spwServerIPDialogBox->get_addressPart1());
    settings.setValue("echoBridgeIP1", this->spwServerIPDialogBox->get_addressPart2());
    settings.setValue("echoBridgeIP2", this->spwServerIPDialogBox->get_addressPart3());
    settings.setValue("echoBridgeIP3", this->spwServerIPDialogBox->get_addressPart4());
}
void activateTCLoopSig();void activateTCLoopSig();
