/*------------------------------------------------------------------------------
--  This file is a part of the LPPMON Software
--  Copyright (C) 20011, Laboratory of Plasma Physics - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Paul Leroy
--                     Mail : paul.leroy@lpp.polytechnique.fr
----------------------------------------------------------------------------*/
#ifndef RMAPPLUGINUI_H
#define RMAPPLUGINUI_H

#include <QWidget>
#include <QPushButton>
#include <QLabel>
#include <QSlider>
#include <QSpinBox>
#include <QGridLayout>
#include <QCheckBox>
#include <QFile>
#include <QTextStream>
#include <QFileDialog>
#include <QDir>
#include <QDialog>
#include <QCloseEvent>
#include <QTabWidget>
#include <QTextEdit>
#include <QRadioButton>
#include <QGroupBox>

#include <parameterdump.h>
#include "rmapoperations.h"

#include "qipdialogbox.h"
#include "tmstatistics.h"
#include "tmechobridge.h"
#include "entermode.h"
#include "lfractions.h"
#include "incomingtmparser.h"

enum selectedBridge{
    selectedBridgeIsUnknown,
    selectedBridgeIsGRESB,
    selectedBridgeIsStarDundee
};

class lfrcontrolPluginUI : public QWidget
{
    Q_OBJECT

public:
    explicit lfrcontrolPluginUI(QWidget *parent = 0);
    ~lfrcontrolPluginUI();

    void onePacketTransmitted( unsigned int nbBytes );

    QPushButton* rmapOpenCommunicationButton;
    QPushButton* rmapCloseCommunicationButton;
    QPushButton* logFileChooseButton;
    QPushButton* gresbStatusQueryRetryButton;
    QPushButton* gresbStatusQueryAbortButton;
    QPushButton* sendCCSDSCommandButton;
    QPushButton* send_TC_LFR_UPDATE_TIME_Button;
    QPushButton* reset_TC_LFR_UPDATE_TIME_Button;
    QPushButton* clearConsoleButton;

    QRadioButton* selectGRESB_BUTTON;
    QRadioButton* selectStarDundee_BUTTON;

    QGroupBox *selection_GROUPBOX;

    QSpinBox* gresbVirtualLinkSpinBox;
//    QSpinBox* spwLinkSpinBox;
    QSpinBox* rmapSourceLogicalAddressSpinBox;
    QSpinBox* rmapTargetLogicalAddressSpinBox;
    QSpinBox* CCSDSTargetLogicalAddressSpinBox;

    QCheckBox *logEnableCheckBox;
    QCheckBox *RMAP_write_verify;
    QCheckBox *RMAP_write_reply;

    QLabel* rmapSendStateLabel;
    QLabel* rmapReceiveStateLabel;
    QLabel* gresbStatusQueryLabel;
    QLabel* gresbStatusQueryDialogLabel;
    QLabel* sendCCSDSCommandLabel;
    QLabel* nbPacketInStore;

    QDialog* gresbStatusQueryDialog;

    QTextEdit* console;
    QTextEdit* consoleSpaceWireServer;

    TMStatistics* tmStatistics;
    LFRActions* lfrActions;

    // TM Echo Bridge
    TMEchoBridge * tmEchoBridge;

    QTcpSocket *socketSpwServer;

    bool islogfileenable();
    void appendToLogFile(const QString & text);
    void closeEvent(QCloseEvent *event);

public slots:
    void chooseLogFile();
    void logFileEnDisable(int state);
    RMAP_command_codes getCommandCode();
    void setSpwServerIP( unsigned char address1, unsigned char address2, unsigned char address3, unsigned char address4 );
    unsigned int getNbTransmittedPackets();
    QList<QVariant> getSWVersion();

private slots:
    void isOpen(bool flag);
    void toggleServer();
    void socket_TMEcho_ServerHasChanged();
    void incomingDataFromSpaceWire();
    void incomingDataFromSpaceWire_alt();
    void connectServer( void );
    void resetstatistics();

signals:
    void connectPortsig(QString PortName,int baudrate);
    void setLogFileName(QString FileName);
    void commandCodeHasChanged(RMAP_command_codes commandCode);
    void bridgeHasChanged(selectedBridge bridge);
    void targetHasChanged(unsigned char target);
    void sourceHasChanged(unsigned char source);
    void sendMessage(QString messageToDisplay);
    void sendPacket(TMPacketToRead*);

private:
    void buildStatisticsLayout( void );
    void buildSelectionLayout( void );
    void disconnectServer( void );
    void initSocketStatesList();
    void readSettings();
    void writeSettings();
    void displayOnConsole(QString message);
    void onePacketReceived( unsigned int nbBytes );

    unsigned char echoBridgeIP0;
    unsigned char echoBridgeIP1;
    unsigned char echoBridgeIP2;
    unsigned char echoBridgeIP3;

    QLabel *gresbBridgeIPLabel;
    QLabel *gresbVirtualLinkLabel;
    QLabel *spwLinkLabel;
    QLabel *rmapSourceLogicalAddressLabel;
    QLabel *rmapTargetLogicalAddressLabel;
    QLabel *logFileName;

    QGridLayout *gresbStatusQueryDialogLayout;
    QGridLayout *generalParameters_LAYOUT;
    QVBoxLayout *selectionLayout;
    QVBoxLayout *layout_TMEcho;
    QVBoxLayout *mainLayout;
    QVBoxLayout *ccsdsLayout;
    QVBoxLayout *consoleLayout;
    QGridLayout *statisticsLayout;

    QTabWidget *spwTabWidget;

    QWidget* bridgeWidget;

    QFile *logFile;
    QTextStream *logFileStrm;
    bool logFileEn;

    QLabel *label_serverTMEcho;
    QIPDialogBox *spwServerIPDialogBox;
    QLabel *label_serverTMEchoPort;
    QSpinBox *spinbox_serverTMEchoPort;
    QPushButton *button_openSocketEchoServer;
    QLabel *label_received;
    QLabel *label_transmitted;
    QLabel *label_bytes;
    QLabel *label_packets;
    QLabel *label_receivedBytes;
    QLabel *label_receivedPackets;
    QLabel *label_transmittedBytes;
    QLabel *label_transmittedPackets;
    QPushButton *button_resetStatistics;

    unsigned int receivedBytes;
    unsigned int receivedPackets;
    unsigned int transmittedBytes;
    unsigned int transmittedPackets;

    QList<QString> socketStates;

    IncomingTMParser *incomingTMParser;

};

#endif // RMAPPLUGINUI_H
